/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.awt.event.KeyEvent;
import java.net.URL;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.carlosbcruz.account.util.AccountAbstractActionDecorator;
import com.carlosbcruz.account.util.AccountMessages;

/**
 * Provider of specif Actions to the com.carlosbcruz.account.control application. The Actions
 * are inner class of AccountActions.
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountActions {

    // Unique actions used by the com.carlosbcruz.account.control application
    private static AccountAbstractActionDecorator fileAction = null;
    private static AccountAbstractActionDecorator importAction = null;
    private static AccountAbstractActionDecorator exportAction = null;
    private static AccountAbstractActionDecorator exitAction = null;

    // Unique actions for Account Internal Frame
    private static AccountAbstractActionDecorator deletetTransactionAction = null;

    // Tree Actions
    private static AccountAbstractActionDecorator insertTransactionAction = null;
    private static AccountAbstractActionDecorator insertCheckAction = null;

    // Manual Transaction Input
    private static AccountAbstractActionDecorator createAction = null;
    private static AccountAbstractActionDecorator insertInQuickListAction = null;
    private static AccountAbstractActionDecorator deleteQuickListAction = null;
    private static AccountAbstractActionDecorator getTransactionFromBankAction = null;
    private static AccountAbstractActionDecorator getBankTypeAction = null;
    private static AccountAbstractActionDecorator bankTypeListAction = null;

    /**
     * Retrieve the action for a specific menu item
     * @param imageIdentification image
     * @param label the label
     * @param mnemonicKey a mnemonic 
     * @param shortDescription a shor description
     * @return the decorator
     */
    private static AccountAbstractActionDecorator getNewAction(
        String imageIdentification, String label, int mnemonicKey,
        String shortDescription) {

        // Retrieve the icon location for the specifc Action
        URL fileURL = AccountActions.class.getResource(AccountMessages
                .getString(imageIdentification));

        // 
        /**
         * Method Inner class that represents the specifc Action
         *
         * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
         */
        class AccountAction extends AccountAbstractActionDecorator {

            AccountAction(String name, Icon icon) {

                super(name, icon);
            }
        }

        AccountAbstractActionDecorator accountAction = new AccountAction(label,
            new ImageIcon(fileURL));

        accountAction.putValue(Action.MNEMONIC_KEY, new Integer(mnemonicKey));

        accountAction.putValue(Action.SHORT_DESCRIPTION, shortDescription);

        return accountAction;

    }

    /**
     * Provides the File Action Instance
     * @return The decorator specific for the File Action
     */
    public static AccountAbstractActionDecorator getFileAction() {

        if (fileAction != null)
            return fileAction;

        return fileAction = getNewAction("fileImage", "File", KeyEvent.VK_F,
            "Handles the Account Storing Files");

    }

    /**
     * Provides the Import Action Instance
     * @return The decorator specific for the Import Action
     */
    public static AccountAbstractActionDecorator getImportAction() {

        if (importAction != null)
            return importAction;

        return importAction = getNewAction("importImage", "Import",
            KeyEvent.VK_I, "Import Account Information from a File");

    }

    /**
     * Provides the Export Action Instance
     * @return The decorator specific for the Export Action
     */
    public static AccountAbstractActionDecorator getExportAction() {

        if (exportAction != null)
            return exportAction;

        return exportAction = getNewAction("exportImage", "Export",
            KeyEvent.VK_E, "Export Account Information to a File");

    }

    /**
     * Provides the Exit Action Instance
     * @return The decorator specific for the Exit Action
     */
    public static AccountAbstractActionDecorator getExitAction() {

        if (exitAction != null)
            return exitAction;

        return exitAction = getNewAction("exitImage", "Exit", KeyEvent.VK_X,
            "Exit the Account Application");
    }

    /**
     * Provides the Delete Transaction Action Instance for Account Internal Frame
     * @return The decorator specific for the Delete Transaction Action
     */
    public static AccountAbstractActionDecorator getDeleteTransactionAction() {

        if (deletetTransactionAction != null)
            return deletetTransactionAction;

        return deletetTransactionAction = getNewAction(
            "deleteTransactionImage", "Delete", KeyEvent.VK_D,
            "Delete the Selected Transaction");

    }

    /**
     * Provides the Insert Transaction Action Instance for Account Internal Frame
     * @return The decorator specific for the Insert Transaction Action
     */
    public static AccountAbstractActionDecorator getInsertTransactionAction() {

        if (insertTransactionAction != null)
            return insertTransactionAction;

        return insertTransactionAction = getNewAction("insertTransactionImage",
            "Insert Transactions", KeyEvent.VK_I,
            "Insert Transactions for the Selected Account");
    }

    /**
     * Provides the Insert Checks Action Instance for Account Internal Frame
     * @return The decorator specific for the Insert Checks Action
     */
    public static AccountAbstractActionDecorator getInsertCheckAction() {

        if (insertCheckAction != null)
            return insertCheckAction;

        return insertCheckAction = getNewAction("insertTransactionImage",
            "Insert Checks", KeyEvent.VK_C,
            "Insert Checks for the Selected Account");
    }

    /**
     * Provides the Create Action on Manual Transaction Input
     * @return The decorator specific for the Create Action
     */
    public static AccountAbstractActionDecorator getCreateTransactionAction() {

        if (createAction != null)
            return createAction;

        return createAction = getNewAction("createActionImage", "Create",
            KeyEvent.VK_C, null);
    }

    /**
     * Provides the Insert In Quick List Action on Manual Transaction Input
     * @return The decorator specific for theInsert In Quick List Action
     */
    public static AccountAbstractActionDecorator getInsertInQuickListAction() {

        if (insertInQuickListAction != null)
            return insertInQuickListAction;

        return insertInQuickListAction = getNewAction(
            "insertQuickListActionImage", "Insert in Quick List",
            KeyEvent.VK_I, "Insert the Description in the Quick List");
    }

    /**
     * Provides the Delete In Quick List Action on Manual Transaction Input
     * @return The decorator specific for the Insert In Quick List Action
     */
    public static AccountAbstractActionDecorator getDeleteQuickListAction() {

        if (deleteQuickListAction != null)
            return deleteQuickListAction;

        return deleteQuickListAction = getNewAction(
            "deleteQuickListActionImage", "Delete Quick List Entry",
            KeyEvent.VK_D, "Delete a Description from the Quick List");
    }

    /**
     * Provides the Action to get Transation from Bank
     * @return The decorator specific for get Transation from Bank
     */
    public static AccountAbstractActionDecorator getTransactionFromBankAction() {

        if (getTransactionFromBankAction != null)
            return getTransactionFromBankAction;

        return getTransactionFromBankAction = getNewAction(
            "getTransactionFromBankImage", "Get Transaction", KeyEvent.VK_G,
            "Get Transaction from Bank Report");
    }

    /**
     * Provides the Action to insert a Bank Type
     * @return The decorator specific for inserting a Bank Type
     */
    public static AccountAbstractActionDecorator getBankTypeAction() {

        if (getBankTypeAction != null)
            return getBankTypeAction;

        return getBankTypeAction = getNewAction("getBankTypeImage",
            "Insert Bank Type", KeyEvent.VK_I,
            "Insert a Bank into a Bank Type List");
    }

    /**
     * Provides the Action to remove a bank type
     * @return The decorator specific for removing a bank type
     */
    public static AccountAbstractActionDecorator getBankTypeListAction() {

        if (bankTypeListAction != null)
            return bankTypeListAction;

        return bankTypeListAction = getNewAction("bankTypeListActionImage",
            "Delete Bank Type", KeyEvent.VK_D,
            "Delete a Bank from a Bank Type List");
    }

}
