/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.awt.Container;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import com.carlosbcruz.account.modelpersistence.AccountFileException;
import com.carlosbcruz.account.modelpersistence.AccountWriter;
import com.carlosbcruz.account.util.AccountAbstractActionDecorator;
import com.carlosbcruz.account.util.AccountObserver;
import com.carlosbcruz.account.util.AccountSubject;

/**
 * Action that works on the Export event
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountEventControlExportAction implements AccountObserver {

    private Container container;
    private AccountAbstractActionDecorator accountAction;
    private AccountJDesktopPane desktopPane;

    /**
     * Constructor that links this class with the event to allow
     * the export function.
     * @param desktopPane The access to the desktop
     * @param accountAction The event that triggers the export event.
     */
    AccountEventControlExportAction(Container container,
        AccountJDesktopPane desktopPane,
        AccountAbstractActionDecorator accountAction) {

        this.container = container;
        this.desktopPane = desktopPane;
        this.accountAction = accountAction;

        accountAction.addObserver(this);
    }

    /* (non-Javadoc)
     * @see com.carlosbcruz.account.util.AccountObserver#update(com.carlosbcruz.account.util.AccountSubject)
     */
    public void update(AccountSubject accountSubject) {

        // Prepare to ask the user for the file
        JFileChooser jFileChooser = new JFileChooser();
        jFileChooser.setMultiSelectionEnabled(false);

        ArrayList fileNamesToRead = new ArrayList();

        int returnValue = jFileChooser.showOpenDialog(container);

        String fileName = null;

        if (returnValue == JFileChooser.APPROVE_OPTION) {

            File files = jFileChooser.getSelectedFile();

            fileName = files.getAbsolutePath();
        } else
            return;

        // Write the file
        try {
            (new AccountWriter()).WriteAllUserAccounts(fileName);
        }
        catch(AccountFileException e) {
            JOptionPane.showMessageDialog(container,
                "Account Application Error - Could not write accounts: "
                    + e.getMessage(), "Account Exception",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
    }
}
