/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * Responds to the Tree Selection Event
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountEventControlLeafTreeSelection implements
    TreeSelectionListener {

    private JTree tree;
    private AccountJDesktopPane desktopPane;

    /**
     * To work on Tree Selection event it is neede to
     * inform the tree and the internal desktop to open an internal window.
     * @param tree Tree that contais the node that was selected
     * @param desktopPane internal desktop that will have the internal frame open
     */
    AccountEventControlLeafTreeSelection(JTree tree,
        AccountJDesktopPane desktopPane) {

        this.tree = tree;
        this.desktopPane = desktopPane;
    }

    /* (non-Javadoc)
     * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
     */
    public void valueChanged(TreeSelectionEvent e) {

        //
        // Request the selected path to the tree
        //

        DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree
                .getLastSelectedPathComponent();

        if (node == null)
            return;

        if (node.isLeaf()) {

            DefaultMutableTreeNode leafParent = (DefaultMutableTreeNode)node
                    .getParent();

            AccountTreeNodeObject leafNode = (AccountTreeNodeObject)node
                    .getUserObject();

            switch (leafNode.getNoteType()) {

                case (AccountTreeNodeObject.ACCOUNT_LEAF):

                {
                    // It is an com.carlosbcruz.account.control leaf
                    AccountTreeNodeObject leafParentObject = (AccountTreeNodeObject)leafParent
                            .getUserObject();
                    String accountId = leafParentObject.getNodeIdentification();
                    int year = Integer.parseInt(leafNode
                            .getNodeIdentification());
                    AccountJInternalAccountFrame internal = AccountJInternalFramesController
                            .getJInternalAccountFrameWithFocus(accountId, year);

                    if (desktopPane.show(internal))
                        // Scroll to the end of the table. Call this method after inserting
                        // the internal frame to the desktop
                        internal.getInternalScrollPane().getVerticalScrollBar()
                                .getModel().setValue(
                                    internal.getInternalScrollPane()
                                            .getVerticalScrollBar().getModel()
                                            .getMaximum());

                    break;
                }
                case (AccountTreeNodeObject.CHECK_LEAF): {
                    // It is a check leaf
                    String groupId = leafNode.getNodeIdentification();
                    AccountJInternalCheckFrame internal = AccountJInternalFramesController
                            .getJInternalCheckFrame(groupId);

                    if (desktopPane.show(internal))
                        // Scroll to the end of the table. Call this method after inserting
                        // the internal frame to the desktop
                        internal.getInternalScrollPane().getVerticalScrollBar()
                                .getModel().setValue(
                                    internal.getInternalScrollPane()
                                            .getVerticalScrollBar().getModel()
                                            .getMaximum());

                    break;
                }
                case (AccountTreeNodeObject.COMMENT_LEAF): {
                    // It is a comment node
                    String commentIdentification = leafNode
                            .getNodeIdentification();

                    AccountJInternalCommentFrame internal = AccountJInternalFramesController
                            .getJInternalCommentFrame(commentIdentification);

                    desktopPane.show(internal);

                    break;

                }
            }
        }
    }
}
