/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.account.control;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import com.carlosbcruz.account.util.AccountObserver;
import com.carlosbcruz.account.util.AccountSubject;

/**
 * Control the popup events from the tree that can reflect over the internal desktop
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountEventControlTreeDesktopPopupListener extends MouseAdapter {

    private JTree tree;
    private final AccountJDesktopPane internalDesktop;

    // Popup menus that can be opened
    private JPopupMenu insertTransactionPopupMenu;
    private JPopupMenu insertChecksPopupMenu;

    // Objects that may be selected
    private String accountIdentificationSelected;
    private String checkIdentificationSelected;

    /**
     * Listen to the insert transaction event and notifies the desktop
     * to show the appropriate internal window.
     * @author carlosbc
     */
    class InsertTransactionActionObserver implements AccountObserver {

        public void update(AccountSubject accountSubject) {

            internalDesktop
                    .show(AccountJInternalFramesController
                            .getJInternalTransactionInputFrame(accountIdentificationSelected));
        }

    }

    /**
     * Listen to the insert check event and notifies the desktop
     * to show the appropriate internal window.
     * @author carlosbc
     */
    class InsertCheckActionObserver implements AccountObserver {

        public void update(AccountSubject accountSubject) {

            internalDesktop.show(AccountJInternalFramesController
                    .getJInternalCheckInputFrame(checkIdentificationSelected));
        }

    }

    /**
     * Constructor that makes the link between the Tree and JDesktopPane through
     * the PopupMenu. 
     * @param tree The tree that can start the popup menu event.
     * @param internalDesktop  The desktop that will show the internal frame
     */
    AccountEventControlTreeDesktopPopupListener(JTree tree,
        AccountJDesktopPane internalDesktop) {

        this.tree = tree;
        this.internalDesktop = internalDesktop;

        // Create the popup menus, transaction and checks
        insertTransactionPopupMenu = new JPopupMenu();
        JMenuItem transactionMenuItem = new JMenuItem(AccountActions
                .getInsertTransactionAction());
        insertTransactionPopupMenu.add(transactionMenuItem);
        AccountActions.getInsertTransactionAction().addObserver(
            new InsertTransactionActionObserver());

        insertChecksPopupMenu = new JPopupMenu();
        JMenuItem checkMenuItem = new JMenuItem(AccountActions
                .getInsertCheckAction());
        insertChecksPopupMenu.add(checkMenuItem);
        AccountActions.getInsertCheckAction().addObserver(
            new InsertCheckActionObserver());
    }

    /* (no-Javadoc)
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e) {

        maybeShowPopup(e);
    }

    /* (no-Javadoc)
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e) {

        maybeShowPopup(e);
    }

    /**
     * Controls the popup menu.
     * @param e the mouse event passed
     */
    private void maybeShowPopup(MouseEvent e) {

        if (e.isPopupTrigger()) {
            int rowSelected = tree.getRowForLocation(e.getX(), e.getY());

            // Only open popup if above an tree node
            if (rowSelected != -1) {

                TreePath path = tree.getPathForRow(rowSelected);
                DefaultMutableTreeNode lastNode = (DefaultMutableTreeNode)path
                        .getLastPathComponent();
                AccountTreeNodeObject accountNode = (AccountTreeNodeObject)lastNode
                        .getUserObject();

                // Show the appropriate popup menu
                if (accountNode.getNoteType() == AccountTreeNodeObject.ACCOUNT_ID) {

                    accountIdentificationSelected = accountNode
                            .getNodeIdentification();

                    insertTransactionPopupMenu.show(e.getComponent(), e.getX(),
                        e.getY());
                    tree.setSelectionInterval(rowSelected, rowSelected);
                }
                if (accountNode.getNoteType() == AccountTreeNodeObject.CHECK_LEAF) {

                    checkIdentificationSelected = accountNode
                            .getNodeIdentification();

                    insertChecksPopupMenu.show(e.getComponent(), e.getX(), e
                            .getY());
                    tree.setSelectionInterval(rowSelected, rowSelected);
                }
            }
        }
    }
}
