/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import com.carlosbcruz.account.dynamicmodel.AccountCurrentAccountSingleton;
import com.carlosbcruz.account.util.AccountObserver;
import com.carlosbcruz.account.util.AccountSubject;

/**
 * Internal Frame that presents an com.carlosbcruz.account.control
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountJInternalAccountFrame extends AccountJInternalFrame {

    //
    // Internal status variables
    //
    private final String accountId;
    private int year;

    //
    // Component references
    //
    private JTable internalTable;
    private JScrollPane internalScrollPane;

    // Pop up to handle transactions
    private JPopupMenu popupTransactions;

    // Internal Model
    private AccountTableModel treeModel;

    /**
     * Construct the internal frame with the grid containning
     * information from the model
     * @param accountIdentification Account ID
     * @param year Account Year
     */
    AccountJInternalAccountFrame(String accountIdentification, int year) {

        // Title
        super(accountIdentification + " - " + year, true, true, true, true);

        // Intenal State
        this.accountId = accountIdentification;
        this.year = year;

        // Initiate the model
        treeModel = new AccountTableModel(accountId, year);

        // Listen to the model
        AccountCurrentAccountSingleton.getAllUserAccounts().getUserAccount(
            accountId).addAccountModelListener(treeModel);

        // If the window is closing than stop listening to the model
        addInternalFrameListener(new InternalFrameAdapter() {

            public void internalFrameClosing(InternalFrameEvent arg0) {

                AccountCurrentAccountSingleton.getAllUserAccounts()
                        .getUserAccount(accountId).removeAccountModelListener(
                            treeModel);
            }
        });

        internalTable = new JTable(treeModel);

        // Update the table if model changes
        treeModel.addTableModelListener(internalTable);

        internalTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
        internalTable.setBackground(new Color(240, 255, 240));

        internalScrollPane = new JScrollPane(internalTable);
        internalTable.doLayout();

        JMenuBar internalMenuBar = new JMenuBar();

        JMenu transactionMenu = new JMenu("Transactions");

        internalMenuBar.add(transactionMenu);

        transactionMenu.add(new JMenuItem(AccountActions
                .getDeleteTransactionAction()));

        setJMenuBar(internalMenuBar);

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(new AccountJInternalAccountToolBar(), BorderLayout.NORTH);

        panel.add(internalScrollPane, BorderLayout.CENTER);

        setContentPane(panel);

        //setContentPane(internalScrollPane);

        // Create a popup menu to allow the insertion of transactions
        popupTransactions = new JPopupMenu();
        JMenuItem menuItem = new JMenuItem(AccountActions
                .getDeleteTransactionAction());

        //	Class to listen to the selection of the insert transaction action
        class DeleteTransactionObserver implements AccountObserver {

            public void update(AccountSubject accountSubject) {

            }
        }

        // Attach the observer to the action
        AccountActions.getDeleteTransactionAction().addObserver(
            new DeleteTransactionObserver());

        popupTransactions.add(menuItem);

        // Class to listen to mouse events and handle the popup
        class PopupListener extends MouseAdapter {

            public void mousePressed(MouseEvent e) {

                maybeShowPopup(e);
            }

            public void mouseReleased(MouseEvent e) {

                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e) {

                if (e.isPopupTrigger()) {
                    popupTransactions
                            .show(e.getComponent(), e.getX(), e.getY());
                    int row = internalTable.rowAtPoint(new Point(e.getX(), e
                            .getY()));
                    internalTable.setRowSelectionInterval(row, row);
                }
            }
        }

        internalTable.addMouseListener(new PopupListener());

    }

    /**
     * Returns the accountId.
     * @return The com.carlosbcruz.account.control identification
     */
    public String getAccountId() {

        return accountId;
    }

    /**
     * Returns the year.
     * @return The frame year
     */
    public int getYear() {

        return year;
    }

    /**
     * Returns the internalScrollPane.
     * @return The scrool pane for the internal pane
     */
    public JScrollPane getInternalScrollPane() {

        return internalScrollPane;
    }

    /**
     * Returns the internalTable.
     * @return The internal table
     */
    public JTable getInternalTable() {

        return internalTable;
    }

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.control.AccountJInternalFrame#getAccountInternalFrameIdentification()
     */
    public String getAccountInternalFrameIdentification() {

        return accountId + year;
    }
}
