/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.awt.Color;

import javax.swing.JScrollPane;
import javax.swing.JTable;

import com.carlosbcruz.account.dynamicmodel.AccountCheck;
import com.carlosbcruz.account.dynamicmodel.AccountCheckGroup;
import com.carlosbcruz.account.dynamicmodel.AccountCurrentAccountSingleton;

/**
 * Internal Frame that presents a check group
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountJInternalCheckFrame extends AccountJInternalFrame {

    //
    // Internal status variables
    //
    private String groupId;

    //
    // Component references
    //
    private JTable internalTable;
    private JScrollPane internalScrollPane;

    /**
     * Construct the internal frame with the grid containning
     * information from the model
     * @param groupId
     */
    AccountJInternalCheckFrame(String groupId) {

        // Title
        super(groupId, true, true, true, true);

        this.groupId = groupId;

        // Get check information from the mode and prepare to
        // show it on GUI
        AccountCheckGroup checkGroup = AccountCurrentAccountSingleton
                .getAllUserAccounts().getCheckGroup(groupId);

        Object data[][] = new Object[checkGroup.getNumberOfChecks()][];

        for (int i = 0; i < checkGroup.getNumberOfChecks(); i++) {
            data[i] = new Object[5];

            AccountCheck check = checkGroup.getCheck(i);

            data[i][0] = String.valueOf(i + 1);
            data[i][1] = check.getCheckIdentification();
            data[i][2] = new Float(check.getValue());
            data[i][3] = check.getDescription();
            data[i][4] = check.getStatus();

        }

        String[] columnNames = {"Sequence", "Identification", "Value",
            "Description", "Status"};

        internalTable = new JTable(data, columnNames);
        internalTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
        internalTable.setBackground(new Color(240, 255, 240));

        internalScrollPane = new JScrollPane(internalTable);
        internalTable.doLayout();

        setContentPane(internalScrollPane);

    }

    /**
     * Returns the groupId.
     * @return The group identification
     */
    public String getGroupId() {

        return groupId;
    }

    /**
     * Returns the internalScrollPane.
     * @return The internal scroll pane
     */
    public JScrollPane getInternalScrollPane() {

        return internalScrollPane;
    }

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.control.AccountJInternalFrame#getAccountInternalFrameIdentification()
     */
    public String getAccountInternalFrameIdentification() {

        return groupId;
    }
}
