/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

/**
 * Constrols the internal frames allocations for the Account Application
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountJInternalFramesController {

    //
    // Control the opened internal frames
    //
    private static Vector internalFrames = new Vector();

    /**
     * Listem to internal frames closing operation
     * @author Carlos Fernando Bella Cruz
     * @version 1
     */
    static class InternalFrameListener extends InternalFrameAdapter {

        public void internalFrameClosing(InternalFrameEvent e) {

            internalFrames.remove(e.getInternalFrame());
        }

    }

    /**
     * Retreive an internal frame based on its identification. If the internal
     * frame is already created than focus it
     * @param frameIdentification The internal frame
     * @return The Internal Frame requested
     */
    private static AccountJInternalFrame seekCreatedJInternalFrame(
        String frameIdentification) {

        // Search for the internal frame
        Enumeration internals = internalFrames.elements();
        AccountJInternalFrame internalFrameFound = null;
        while (internals.hasMoreElements()) {
            AccountJInternalFrame internalFrame = (AccountJInternalFrame)internals
                    .nextElement();

            if (internalFrame.getAccountInternalFrameIdentification().equals(
                frameIdentification))
                internalFrameFound = internalFrame;
        }

        return internalFrameFound;
    }

    /**
     * Request focus to the internal frame specified
     * @param frame The frame to get focus
     */
    private static void requestFocus(AccountJInternalFrame frame) {

        if (frame.isIcon())
            frame.tryToDeiconify();
        else
            frame.tryToGetFocus();

    }

    /**
     * Retrieve the internal frame if it was created otherwise create a new one
     * @param accountId the com.carlosbcruz.account.control id to be used
     * @param year the year to be created
     * @return the internal frame requested
     */
    private static AccountJInternalAccountFrame getJInternalAccountFrame(
        String accountId, int year) {

        AccountJInternalFrame internalFrame = seekCreatedJInternalFrame(accountId
            + year);

        if (internalFrame == null) {

            AccountJInternalAccountFrame internalAccountFrame = new AccountJInternalAccountFrame(
                accountId, year);

            internalAccountFrame
                    .addInternalFrameListener(new InternalFrameListener());

            internalFrames.add(internalAccountFrame);

            return internalAccountFrame;
        }

        return (AccountJInternalAccountFrame)internalFrame;

    }

    /**
     * Retreive the Account Internal Frame corresponding with
     * the com.carlosbcruz.account.control id and year and request the focus
     * @param accountId The internal com.carlosbcruz.account.control identification
     * @param year The year of the data inside
     * @return AccountJInternaAccountlFrame the Internal Frame requested
     */
    public static AccountJInternalAccountFrame getJInternalAccountFrameWithFocus(
        String accountId, int year) {

        AccountJInternalFrame internalFrame = getJInternalAccountFrame(
            accountId, year);

        requestFocus(internalFrame);

        return (AccountJInternalAccountFrame)internalFrame;
    }

    /**
     * Get the internal frame corresponding to the check group identification
     * @param checkGroupId The check group identification
     * @return AccountJInternaCheckFrame the internanal frame requested
     */
    public static AccountJInternalCheckFrame getJInternalCheckFrame(
        String checkGroupId) {

        AccountJInternalFrame internalFrame = seekCreatedJInternalFrame(checkGroupId);

        if (internalFrame == null) {

            AccountJInternalCheckFrame internalCheckFrame = new AccountJInternalCheckFrame(
                checkGroupId);

            internalCheckFrame
                    .addInternalFrameListener(new InternalFrameListener());

            internalFrames.add(internalCheckFrame);

            return internalCheckFrame;
        }

        return (AccountJInternalCheckFrame)internalFrame;

    }

    /**
     * Get the internal frame corresponding to the comment identification
     * @param commentIdentification The comment indentification
     * @return The internanal frame requested
     */
    public static AccountJInternalCommentFrame getJInternalCommentFrame(
        String commentIdentification) {

        AccountJInternalFrame internalFrame = seekCreatedJInternalFrame(commentIdentification);

        if (internalFrame == null) {

            AccountJInternalCommentFrame internalCommentFrame = new AccountJInternalCommentFrame(
                commentIdentification);

            internalCommentFrame
                    .addInternalFrameListener(new InternalFrameListener());

            internalFrames.add(internalCommentFrame);

            return internalCommentFrame;
        }

        return (AccountJInternalCommentFrame)internalFrame;
    }

    /**
     * Get the internal frame corresponding to the insertion of transaction
     * @param accountIdentification The Account Identification
     * @return The internanal frame requested
     */
    public static AccountJInternalTransactionInput getJInternalTransactionInputFrame(
        String accountIdentification) {

        AccountJInternalFrame internalFrame = seekCreatedJInternalFrame(accountIdentification
            + AccountJInternalTransactionInput.ID_INSERT_APPEND_IDENTIFICATION);

        if (internalFrame == null) {

            AccountJInternalTransactionInput internalTransactionInputFrame = new AccountJInternalTransactionInput(
                accountIdentification);

            internalTransactionInputFrame
                    .addInternalFrameListener(new InternalFrameListener());

            internalFrames.add(internalTransactionInputFrame);

            return internalTransactionInputFrame;
        }

        return (AccountJInternalTransactionInput)internalFrame;
    }

    /**
     * Get the internal frame corresponding to the insertion of checks
     * @param checkIdentification The Account Identification
     * @return The internanal frame requested
     */
    public static AccountJInternalCheckInput getJInternalCheckInputFrame(
        String checkIdentification) {

        AccountJInternalFrame internalFrame = seekCreatedJInternalFrame(checkIdentification
            + AccountJInternalCheckInput.ID_INSERT_APPEND_IDENTIFICATION);

        if (internalFrame == null) {

            AccountJInternalCheckInput internalCheckInputFrame = new AccountJInternalCheckInput(
                checkIdentification);

            internalCheckInputFrame
                    .addInternalFrameListener(new InternalFrameListener());

            internalFrames.add(internalCheckInputFrame);

            return internalCheckInputFrame;
        }

        return (AccountJInternalCheckInput)internalFrame;
    }

}
