/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.BadLocationException;

import com.carlosbcruz.account.dynamicmodel.AccountCurrentAccountSingleton;
import com.carlosbcruz.account.dynamicmodel.AccountTransaction;
import com.carlosbcruz.account.modeltotals.AccountCalculations;
import com.carlosbcruz.account.modeltotals.AccountTransactionTotal;
import com.carlosbcruz.account.util.AccountObserver;
import com.carlosbcruz.account.util.AccountProperties;
import com.carlosbcruz.account.util.AccountSubject;
import com.carlosbcruz.account.util.AccountValues;
import com.carlosbcruz.account.util.NumberNotFoundException;

/**
 * Provide the functionality of insertion of transactions to a specific com.carlosbcruz.account.control
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountJInternalTransactionInput extends AccountJInternalFrame {

    // The identification of the transaction internal frame is the com.carlosbcruz.account.control identification
    // appended with the " Input" string.
    public static final String ID_INSERT_APPEND_IDENTIFICATION = " Input";

    // This form belongs to the following com.carlosbcruz.account.control ID.
    private String accountId;

    // Input Fields that contains the main informations for the transaction
    private JTextField dateField;
    private JTextField valueField;
    private JTextField descriptionField;
    private JRadioButton widthdrawButton;
    private JRadioButton depositButton;
    private JRadioButton savingsButton;
    private JRadioButton savingsToAccountButton;
    private JRadioButton accountToSavingsButton;

    // Control the last selected item on the quick list.
    private static int lastQuickListSelected = -1;
    // Control the last selected item on the bank type list
    private static int lastBankTypeListSelected = -1;

    // Allow descriptions to be inserted in the quick list fild	
    private JButton insertQuickList;
    private JLabel status;
    private JButton createButton;

    // Text editor to hold the bank report
    private JEditorPane bankEditor;
    private JScrollPane internalBankScrollPane;
    private QuickListModel quickListModel = new QuickListModel();

    // Model of the bank formats available
    private BankListModel bankListModel = new BankListModel();
    private final JList bankTypeList = new JList();

    // Totals information	
    private JLabel dateValue = new JLabel("DD/MM/YYYYY");
    private JLabel lastTransactionValue = new JLabel("0");
    private JLabel accountValue = new JLabel("0");
    private JLabel savingsValue = new JLabel("0");
    private JLabel descriptionValue = new JLabel("No Description");

    /**
     * Constructor that receives the com.carlosbcruz.account.control to control the insertion of transactions
     * @param accountId The identification of the transaction
     */
    AccountJInternalTransactionInput(String accountId) {

        // Title
        super(accountId + ID_INSERT_APPEND_IDENTIFICATION, true, true, true,
            true);

        this.accountId = accountId;

        setContentPane(constructManualTab());

    }

    /**
     * Maps the model of quicklist to the Account Property files. 
     * @author carlos
     */
    class QuickListModel implements ListModel {

        Vector modelChangeListeners = new Vector();

        /* (no-Javadoc)
         * @see javax.swing.ListModel#getSize()
         */
        public int getSize() {

            // Return how many propertyes here are for the quick list group
            return AccountProperties.getGroupProperty(
                AccountProperties.QUICK_DESCRIPTION_GROUP).size();
        }

        /* (no-Javadoc)
         * @see javax.swing.ListModel#getElementAt(int)
         */
        public Object getElementAt(int arg0) {

            // Return property by position
            return AccountProperties.getGroupProperty(
                AccountProperties.QUICK_DESCRIPTION_GROUP, arg0);
        }

        /**
         * Inform all the listeners that the list were changed. It is saying
         * tha all the elements should be refreshed.
         */
        public void notifyChange() {

            int numberOfElements = AccountProperties.getGroupProperty(
                AccountProperties.QUICK_DESCRIPTION_GROUP).size();

            Enumeration e = modelChangeListeners.elements();
            while (e.hasMoreElements()) {
                ListDataListener listener = (ListDataListener)e.nextElement();

                listener.contentsChanged(new ListDataEvent(this,
                    ListDataEvent.INTERVAL_ADDED, 0, numberOfElements));
            }
        }

        /* (no-Javadoc)
         * @see javax.swing.ListModel#addListDataListener(javax.swing.event.ListDataListener)
         */
        public void addListDataListener(ListDataListener arg0) {

            modelChangeListeners.add(arg0);
        }

        /* (no-Javadoc)
         * @see javax.swing.ListModel#removeListDataListener(javax.swing.event.ListDataListener)
         */
        public void removeListDataListener(ListDataListener arg0) {

            modelChangeListeners.remove(arg0);
        }

    };

    /**
     * Maps the model of a bank list to the Account Property files. 
     * @author carlos
     */
    class BankListModel implements ListModel {

        Vector modelChangeListeners = new Vector();

        /* (no-Javadoc)
         * @see javax.swing.ListModel#getSize()
         */
        public int getSize() {

            // Return how many propertyes here are for the quick list group
            return AccountProperties.getGroupProperty(
                AccountProperties.BANK_TYPE_DESCRIPTION_GROUP).size();
        }

        /* (no-Javadoc)
         * @see javax.swing.ListModel#getElementAt(int)
         */
        public Object getElementAt(int arg0) {

            String bankTypeProperty = AccountProperties.getGroupProperty(
                AccountProperties.BANK_TYPE_DESCRIPTION_GROUP, arg0);

            int index = bankTypeProperty.indexOf("#");

            // Check errors
            if (index == -1) {
                return "Properties Error";
            }
            // Return property by position
            return bankTypeProperty.substring(0, index);
        }

        /**
         * Inform all the listeners that the list were changed. It is saying
         * tha all the elements should be refreshed.
         */
        public void notifyChange() {

            int numberOfElements = AccountProperties.getGroupProperty(
                AccountProperties.BANK_TYPE_DESCRIPTION_GROUP).size();

            Enumeration e = modelChangeListeners.elements();
            while (e.hasMoreElements()) {
                ListDataListener listener = (ListDataListener)e.nextElement();

                listener.contentsChanged(new ListDataEvent(this,
                    ListDataEvent.INTERVAL_ADDED, 0, numberOfElements));
            }
        }

        /* (no-Javadoc)
         * @see javax.swing.ListModel#addListDataListener(javax.swing.event.ListDataListener)
         */
        public void addListDataListener(ListDataListener arg0) {

            modelChangeListeners.add(arg0);
        }

        /* (no-Javadoc)
         * @see javax.swing.ListModel#removeListDataListener(javax.swing.event.ListDataListener)
         */
        public void removeListDataListener(ListDataListener arg0) {

            modelChangeListeners.remove(arg0);
        }

    };

    /**
     * Construct the main transaction input form
     * @return The panela required
     */
    private JSplitPane constructManualTab() {

        // Create the information panel
        GridBagLayout informationGridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        JPanel informationPanel = new JPanel(informationGridBagLayout);
        TitledBorder inputBorder = BorderFactory
                .createTitledBorder("Transaction Manual Input");
        informationPanel.setBorder(inputBorder);

        // Insert the fields
        JLabel dateLabel = new JLabel("Date:");
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(dateLabel, gridBagConstraints);
        informationPanel.add(dateLabel);

        dateField = new JTextField(8);
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(dateField, gridBagConstraints);
        informationPanel.add(dateField);

        JLabel valueLabel = new JLabel("Value:");
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(valueLabel, gridBagConstraints);
        informationPanel.add(valueLabel);

        valueField = new JTextField(8);
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(valueField, gridBagConstraints);
        informationPanel.add(valueField);

        JLabel descriptionLabel = new JLabel("Description:");
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(descriptionLabel,
            gridBagConstraints);
        informationPanel.add(descriptionLabel);

        descriptionField = new JTextField(20);
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(descriptionField,
            gridBagConstraints);
        informationPanel.add(descriptionField);

        insertQuickList = new JButton(AccountActions
                .getInsertInQuickListAction());
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        informationGridBagLayout.setConstraints(insertQuickList,
            gridBagConstraints);
        informationPanel.add(insertQuickList);

        // Observe if the insert to quick list button was pressed
        AccountActions.getInsertInQuickListAction().addObserver(
            new AccountObserver() {

                public void update(AccountSubject accountSubject) {

                    // Only allows descriptions non empty
                    String description = descriptionField.getText();
                    if (description == null)
                        description = new String();
                    if (description.equals(""))
                        return;

                    AccountProperties.addGroupProperty(
                        AccountProperties.QUICK_DESCRIPTION_GROUP, description);

                    quickListModel.notifyChange();

                }

            });

        // Insert transaction type panel
        JPanel transactionTypePanel = new JPanel(new GridLayout(0, 1));
        TitledBorder transactionTypeBorder = BorderFactory
                .createTitledBorder("Transaction Type");
        transactionTypePanel.setBorder(transactionTypeBorder);

        widthdrawButton = new JRadioButton("Widthdraw");
        depositButton = new JRadioButton("Account Deposit");
        savingsButton = new JRadioButton("Savings Deposit");
        savingsToAccountButton = new JRadioButton("Savings To Account");
        accountToSavingsButton = new JRadioButton("Account To Savings");

        widthdrawButton.setSelected(true);

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(widthdrawButton);
        buttonGroup.add(depositButton);
        buttonGroup.add(savingsButton);
        buttonGroup.add(savingsToAccountButton);
        buttonGroup.add(accountToSavingsButton);

        transactionTypePanel.add(widthdrawButton);
        transactionTypePanel.add(depositButton);
        transactionTypePanel.add(savingsButton);
        transactionTypePanel.add(savingsToAccountButton);
        transactionTypePanel.add(accountToSavingsButton);

        JPanel controlPanel = new JPanel(new BorderLayout());

        // Create the status bar on the window button and the create transactio button
        status = new JLabel();
        controlPanel.add(status, BorderLayout.WEST);
        createButton = new JButton(AccountActions.getCreateTransactionAction());
        controlPanel.add(createButton, BorderLayout.EAST);

        // Observe if the create transaction button was pressed
        AccountActions.getCreateTransactionAction().addObserver(
            new AccountObserver() {

                public void update(AccountSubject accountSubject) {

                    // Only insert the transaction if all the consistencies
                    // were checkd
                    if (checkValues() == true) {

                        StringTokenizer tokenizer = new StringTokenizer(
                            dateField.getText(), "/");
                        String day = tokenizer.nextToken();
                        String month = tokenizer.nextToken();
                        String year = tokenizer.nextToken();

                        String valueContent = valueField.getText().trim();

                        NumberFormat format = NumberFormat.getNumberInstance();
                        format.setMaximumFractionDigits(2);

                        Number valueTest = null;
                        try {
                            valueTest = format.parse(valueContent);
                        }
                        catch(ParseException e1) {
                            e1.printStackTrace();
                            return;
                        }

                        AccountTransaction transaction = new AccountTransaction();
                        transaction.setDay(Integer.parseInt(day));
                        transaction.setMonth(Integer.parseInt(month));
                        transaction.setYear(Integer.parseInt(year));
                        transaction.setDescription(descriptionField.getText());
                        transaction.setValue(valueTest.floatValue());

                        if (widthdrawButton.isSelected())
                            transaction.setType(AccountTransaction.WITHDRAW);
                        if (depositButton.isSelected())
                            transaction
                                    .setType(AccountTransaction.DEPOSIT_CURRENT_ACCOUNT);
                        if (savingsButton.isSelected())
                            transaction
                                    .setType(AccountTransaction.DEPOSIT_SAVINGS_ACCOUNT);
                        if (savingsToAccountButton.isSelected())
                            transaction
                                    .setType(AccountTransaction.MOVE_FROM_SAVINGS_TO_CURRENT_ACCOUNT);
                        if (accountToSavingsButton.isSelected())
                            transaction
                                    .setType(AccountTransaction.MOVE_FROM_CURRENT_ACCOUNT_TO_SAVINGS);

                        AccountCurrentAccountSingleton.getAllUserAccounts()
                                .getUserAccount(accountId).addTransaction(
                                    transaction);

                        AccountTransactionTotal totals[];

                        int howManuYear = AccountCurrentAccountSingleton
                                .getAllUserAccounts().getUserAccount(accountId)
                                .getNumberOfYears();
                        int targetYear = AccountCurrentAccountSingleton
                                .getAllUserAccounts().getUserAccount(accountId)
                                .getYearByPosition(howManuYear);

                        totals = AccountCalculations.getAccountTotals(
                            accountId, targetYear);

                        AccountTransactionTotal lastTransaction = totals[totals.length - 1];

                        dateValue.setText(dateField.getText());
                        lastTransactionValue.setText(valueContent);
                        accountValue.setText(String.valueOf(lastTransaction
                                .getAccountTotal()));
                        savingsValue.setText(String.valueOf(lastTransaction
                                .getSavingsTotal()));
                        descriptionValue.setText(descriptionField.getText());

                        // Clean the fields
                        status.setText("Transaction "
                            + descriptionField.getText() + " Created");
                        valueField.setText("");
                        descriptionField.setText("");

                    }

                }
            });

        // Create the Quick List

        final JList quickList = new JList(quickListModel);
        DefaultListSelectionModel selectionModel = new DefaultListSelectionModel();
        selectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        quickList.setSelectionModel(selectionModel);

        // Set the description on input field if a selection was made.
        selectionModel.addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent arg0) {

                int element = quickList.getSelectedIndex();
                if (element < 0)
                    return;

                descriptionField
                        .setText((String)AccountProperties.getGroupProperty(
                            AccountProperties.QUICK_DESCRIPTION_GROUP, element));

            }
        });

        // Construct the panel that will hold the quick list.
        JPanel quickListPanel = new JPanel(new BorderLayout());
        TitledBorder quickListBorder = BorderFactory
                .createTitledBorder("Quick List");
        quickListPanel.setBorder(quickListBorder);
        quickListPanel.add(new JScrollPane(quickList), BorderLayout.CENTER);

        // Create a popup menu on quick list to allow the deletion of entries.
        final JPopupMenu quickListPopup = new JPopupMenu();
        quickListPopup.add(AccountActions.getDeleteQuickListAction());
        AccountActions.getDeleteQuickListAction().addObserver(
            new AccountObserver() {

                public void update(AccountSubject accountSubject) {

                    int element = quickList.getSelectedIndex();
                    if (element < 0)
                        return;

                    AccountProperties.removeGroupKeyPropertyByPosition(
                        AccountProperties.QUICK_DESCRIPTION_GROUP, element);

                    quickListModel.notifyChange();
                }
            });

        // Class to listen to mouse events and handle the popup
        class QuickListPopupListener extends MouseAdapter {

            public void mousePressed(MouseEvent e) {

                // See if it was clicked on a item that is already selected
                // if yes than unselect it.
                int row = quickList.locationToIndex(new Point(e.getX(), e
                        .getY()));

                // If it was clicked on the same item
                if (lastQuickListSelected == row) {

                    // If it is not selected, than select it
                    if (quickList.isSelectionEmpty()) {

                        quickList.setSelectedIndex(row);
                        lastQuickListSelected = row;

                    } else {

                        // Clear the selection
                        quickList.clearSelection();
                        lastQuickListSelected = -1;

                    }

                } else {

                    // It was clicked on another item, than select it.
                    quickList.setSelectedIndex(row);
                    lastQuickListSelected = row;

                }

                maybeShowPopup(e);

            }

            public void mouseReleased(MouseEvent e) {

                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e) {

                int row = quickList.locationToIndex(new Point(e.getX(), e
                        .getY()));

                if (e.isPopupTrigger()) {

                    if (row >= 0) {

                        quickListPopup.show(e.getComponent(), e.getX(), e
                                .getY());

                        quickList.setSelectedIndex(row);

                    }
                }
            }

        }

        quickList.addMouseListener(new QuickListPopupListener());
        quickList.add(quickListPopup);

        // Construct the main bank panel

        JPanel bankPanel = new JPanel(new BorderLayout());

        // Construct the Editor
        JPanel bankEditorPanel = new JPanel(new BorderLayout());
        TitledBorder bankBorder = BorderFactory
                .createTitledBorder("Bank Input");
        bankEditorPanel.setBorder(bankBorder);

        bankEditor = new JEditorPane("text/plain", "");
        internalBankScrollPane = new JScrollPane(bankEditor);

        bankEditorPanel.add(internalBankScrollPane, BorderLayout.CENTER);

        JButton retrieveLineFromBankButton = new JButton(AccountActions
                .getTransactionFromBankAction());

        bankEditorPanel.add(retrieveLineFromBankButton, BorderLayout.SOUTH);

        bankPanel.add(bankEditorPanel, BorderLayout.CENTER);

        // If a button to retrieve the line from bank report is pressed
        AccountActions.getTransactionFromBankAction().addObserver(
            new AccountObserver() {

                public void update(AccountSubject accountSubject) {

                    try {

                        // There must be a bank selected
                        int bankTypeSelected = bankTypeList.getSelectedIndex();
                        if (bankTypeSelected == -1) {
                            showErrorMessage("Select a Bank Type first");
                            return;
                        }

                        // Get the length of the first line
                        int numberOfCharsToBeRemoved = getBankLineSize(bankEditor
                                .getDocument().getText(0,
                                    bankEditor.getDocument().getLength()));

                        // Instantiate the bank class dinamically
                        String bankTypeProperty = AccountProperties
                                .getGroupProperty(
                                    AccountProperties.BANK_TYPE_DESCRIPTION_GROUP,
                                    bankTypeSelected);

                        int position = bankTypeProperty.indexOf("#");

                        String classIdentification = bankTypeProperty
                                .substring(position + 1, bankTypeProperty
                                        .length());

                        AccountParser bankParser = null;
                        try {
                            bankParser = (AccountParser)Class.forName(
                                classIdentification).newInstance();
                        }
                        catch(InstantiationException e) {
                            showErrorMessage("It was not possible to instantiate the class: "
                                + classIdentification);
                            return;
                        }
                        catch(IllegalAccessException e) {
                            showErrorMessage("It was not possible to access the class: "
                                + classIdentification);
                            return;
                        }
                        catch(ClassNotFoundException e) {
                            showErrorMessage("The class: "
                                + classIdentification + " was not found.");
                            return;
                        }
                        catch(Throwable e) {
                            showErrorMessage("Error loading class: "
                                + classIdentification);
                            return;
                        }

                        AccountTransactionBean bean = bankParser
                                .parseFirstTransactionLine(bankEditor
                                        .getDocument().getText(
                                            0,
                                            bankEditor.getDocument()
                                                    .getLength()));

                        dateField.setText(bean.getDay() + "/" + bean.getMonth()
                            + "/" + bean.getYear());
                        valueField.setText(bean.getValue());
                        descriptionField.setText(bean.getDescription());

                        if (bean.isWithdraw())
                            widthdrawButton.setSelected(true);
                        else
                            depositButton.setSelected(true);

                        bankEditor.getDocument().remove(0,
                            numberOfCharsToBeRemoved);

                    }
                    catch(BadLocationException e) {

                        e.printStackTrace();
                    }
                    catch(AccountException e) { // Do nothing
                    }

                }
            });

        // Build the bank type selectin list
        bankTypeList.setModel(bankListModel);

        JPanel bankTypePanel = new JPanel(new BorderLayout());
        TitledBorder bankTypeBorder = BorderFactory
                .createTitledBorder("Bank Types");
        bankTypePanel.setBorder(bankTypeBorder);
        bankTypePanel.add(new JScrollPane(bankTypeList), BorderLayout.CENTER);

        GridBagLayout bankTypeGridBagLayout = new GridBagLayout();
        JPanel bankTypeInputPanel = new JPanel(bankTypeGridBagLayout);

        JLabel bankTitle = new JLabel("Bank");
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        bankTypeGridBagLayout.setConstraints(bankTitle, gridBagConstraints);
        bankTypeInputPanel.add(bankTitle);

        final JTextField titleInput = new JTextField(8);
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        bankTypeGridBagLayout.setConstraints(titleInput, gridBagConstraints);
        bankTypeInputPanel.add(titleInput);

        JLabel className = new JLabel("Class");
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        bankTypeGridBagLayout.setConstraints(className, gridBagConstraints);
        bankTypeInputPanel.add(className);

        final JTextField classInput = new JTextField(8);
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        bankTypeGridBagLayout.setConstraints(classInput, gridBagConstraints);
        bankTypeInputPanel.add(classInput);

        JButton insertBankType = new JButton(AccountActions.getBankTypeAction());
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 4;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        bankTypeGridBagLayout
                .setConstraints(insertBankType, gridBagConstraints);
        bankTypeInputPanel.add(insertBankType);

        AccountActions.getBankTypeAction().addObserver(new AccountObserver() {

            public void update(AccountSubject accountSubject) {

                String bankName = titleInput.getText();
                if (bankName == null)
                    bankName = new String();

                String classIdentification = classInput.getText();
                if (classIdentification == null)
                    classIdentification = new String();

                if (bankName.equals("")) {
                    showErrorMessage("Specify the Bank Name first");
                    return;
                }

                if (classIdentification.equals("")) {
                    showErrorMessage("Specify Parser Class first");
                    return;
                }

                if (bankName.indexOf("#") != -1) {
                    showErrorMessage("The Bank Name can not contains # symble");
                    return;
                }

                try {
                    AccountParser bankParser = (AccountParser)Class.forName(
                        classIdentification).newInstance();
                }
                catch(InstantiationException e) {
                    showErrorMessage("It was not possible to instantiate the class: "
                        + classIdentification);
                    return;
                }
                catch(IllegalAccessException e) {
                    showErrorMessage("It was not possible to access the class: "
                        + classIdentification);
                    return;
                }
                catch(ClassNotFoundException e) {
                    showErrorMessage("The class: " + classIdentification
                        + " was not found.");
                    return;
                }
                catch(Throwable e) {
                    showErrorMessage("Error loading class: "
                        + classIdentification);
                    return;
                }

                AccountProperties.addGroupProperty(
                    AccountProperties.BANK_TYPE_DESCRIPTION_GROUP, bankName
                        + "#" + classIdentification);
                bankListModel.notifyChange();
            }

        });

        bankTypePanel.add(bankTypeInputPanel, BorderLayout.SOUTH);

        bankPanel.add(bankTypePanel, BorderLayout.EAST);

        // Create the last transaction information

        GridBagLayout totalsGridBag = new GridBagLayout();
        JPanel totalPanels = new JPanel(totalsGridBag);
        TitledBorder totalsPanelsBorder = BorderFactory
                .createTitledBorder("Last Transaction");
        totalPanels.setBorder(totalsPanelsBorder);

        Font lastTransactionFont = new Font(null, Font.BOLD, 14);
        Color lastTransactionColor = Color.black;

        JLabel dateLastTransactionLabel = new JLabel("Date");
        dateLastTransactionLabel.setFont(lastTransactionFont);
        dateLastTransactionLabel.setForeground(lastTransactionColor);
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(dateLastTransactionLabel,
            gridBagConstraints);
        totalPanels.add(dateLastTransactionLabel);

        JLabel valueLastTransactionLabel = new JLabel("Value");
        valueLastTransactionLabel.setFont(lastTransactionFont);
        valueLastTransactionLabel.setForeground(lastTransactionColor);
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(valueLastTransactionLabel,
            gridBagConstraints);
        totalPanels.add(valueLastTransactionLabel);

        JLabel accountLastTransactionLabel = new JLabel("Account");
        accountLastTransactionLabel.setFont(lastTransactionFont);
        accountLastTransactionLabel.setForeground(lastTransactionColor);
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(accountLastTransactionLabel,
            gridBagConstraints);
        totalPanels.add(accountLastTransactionLabel);

        JLabel savingsLastTransactionLabel = new JLabel("Savings");
        savingsLastTransactionLabel.setFont(lastTransactionFont);
        savingsLastTransactionLabel.setForeground(lastTransactionColor);
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(savingsLastTransactionLabel,
            gridBagConstraints);
        totalPanels.add(savingsLastTransactionLabel);

        JLabel descriptionLastTransactionLabel = new JLabel("Description");
        descriptionLastTransactionLabel.setFont(lastTransactionFont);
        descriptionLastTransactionLabel.setForeground(lastTransactionColor);
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(descriptionLastTransactionLabel,
            gridBagConstraints);
        totalPanels.add(descriptionLastTransactionLabel);

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(dateValue, gridBagConstraints);
        totalPanels.add(dateValue);

        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(lastTransactionValue, gridBagConstraints);
        totalPanels.add(lastTransactionValue);

        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(accountValue, gridBagConstraints);
        totalPanels.add(accountValue);

        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(savingsValue, gridBagConstraints);
        totalPanels.add(savingsValue);

        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.ipadx = 5;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        totalsGridBag.setConstraints(descriptionValue, gridBagConstraints);
        totalPanels.add(descriptionValue);

        // Create a popup menu on bank type allow the deletion of entries.
        final JPopupMenu bankTypeListPopup = new JPopupMenu();
        bankTypeListPopup.add(AccountActions.getBankTypeListAction());
        AccountActions.getBankTypeListAction().addObserver(
            new AccountObserver() {

                public void update(AccountSubject accountSubject) {

                    int element = bankTypeList.getSelectedIndex();
                    if (element < 0)
                        return;

                    AccountProperties.removeGroupKeyPropertyByPosition(
                        AccountProperties.BANK_TYPE_DESCRIPTION_GROUP, element);
                    bankListModel.notifyChange();
                }
            });

        // Class to listen to mouse events and handle the popup
        class BankTypeListPopupListener extends MouseAdapter {

            public void mousePressed(MouseEvent e) {

                // See if it was clicked on a item that is already selected
                // if yes than unselect it.
                int row = bankTypeList.locationToIndex(new Point(e.getX(), e
                        .getY()));

                // If it was clicked on the same item
                if (lastBankTypeListSelected == row) {

                    // If it is not selected, than select it
                    if (bankTypeList.isSelectionEmpty()) {

                        bankTypeList.setSelectedIndex(row);
                        lastBankTypeListSelected = row;

                    } else {

                        // Clear the selection
                        bankTypeList.clearSelection();
                        lastBankTypeListSelected = -1;

                    }

                } else {

                    // It was clicked on another item, than select it.
                    bankTypeList.setSelectedIndex(row);
                    lastBankTypeListSelected = row;

                }

                maybeShowPopup(e);

            }

            public void mouseReleased(MouseEvent e) {

                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e) {

                int row = bankTypeList.locationToIndex(new Point(e.getX(), e
                        .getY()));

                if (e.isPopupTrigger()) {

                    if (row >= 0) {

                        bankTypeListPopup.show(e.getComponent(), e.getX(), e
                                .getY());

                        bankTypeList.setSelectedIndex(row);

                    }
                }
            }

        }

        bankTypeList.addMouseListener(new BankTypeListPopupListener());
        bankTypeList.add(bankTypeListPopup);

        // Join all the panels above in a main manual input panel
        JPanel manualMainContainer = new JPanel(new BorderLayout());
        manualMainContainer.add(totalPanels, BorderLayout.NORTH);
        manualMainContainer.add(informationPanel, BorderLayout.WEST);
        manualMainContainer.add(quickListPanel, BorderLayout.CENTER);
        manualMainContainer.add(transactionTypePanel, BorderLayout.EAST);
        manualMainContainer.add(controlPanel, BorderLayout.SOUTH);
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
            bankPanel, manualMainContainer);
        splitPane.setDividerLocation(150);
        return splitPane;
    }

    /**
     * Extract a bank line size from the text with the bank information
     * @param bankText The bank text
     * @return the position of the end of the line that ware readed
     * @throws AccountException
     */
    private int getBankLineSize(String bankText) throws AccountException {

        // See if is not empty
        if (bankText == null)
            bankText = new String();
        if (bankText.equals("")) {
            throw new AccountException(
                "There is no text in the Bank Input to be parsed");
        }

        // Seek for a end of line
        int firstEndOfLine = -1;
        for (int i = 0; i < bankText.length(); i++) {
            char auxiliarChar = bankText.charAt(i);
            if (auxiliarChar == '\n') {

                firstEndOfLine = i + 1;
                break;
            }
        }

        if (firstEndOfLine == -1)
            firstEndOfLine = bankText.length();

        return firstEndOfLine;
    }

    /**
     * Show a error message  
     * @param message The message to be presented
     */
    private void showErrorMessage(String message) {

        JOptionPane.showMessageDialog(this, message);
    }

    /**
     * Verify if all the values from the manual pane is correct. If any value is
     * wrong than open a modal window informing the problem.
     * @return true if all the values are ok and false otherwise
     */
    private boolean checkValues() {

        // Check if dat is empty
        String date = dateField.getText();
        if (date == null)
            date = new String();
        if (date.equals("")) {
            showErrorMessage("The Date field is empty");
            return false;
        }

        // Check if the date ia valid date format
        int day, month, year;
        try {

            StringTokenizer tokenizer = new StringTokenizer(
                dateField.getText(), "/");
            String dayAuxiliar = tokenizer.nextToken();
            String monthAuxiliar = tokenizer.nextToken();
            String yearAuxiliar = tokenizer.nextToken();
            day = Integer.parseInt(dayAuxiliar);
            month = Integer.parseInt(monthAuxiliar);
            year = Integer.parseInt(yearAuxiliar);

        }
        catch(Exception e) {
            showErrorMessage("The Date field not well formatted, use dd/mm/yyyy.");
            return false;
        }

        // Check if the date is a valid calendar date
        Calendar inputCalendar = Calendar.getInstance();
        try {

            inputCalendar.setLenient(false);
            inputCalendar.set(year, month - 1, day, 0, 0);

            // this will cause an exception if the date is wrong
            inputCalendar.get(java.util.Calendar.DAY_OF_MONTH);

        }
        catch(Exception e) {
            showErrorMessage("The Date is not a valid calendar date");
            return false;
        }

        // Get the date of the last transaction
        int numberOfTransactions = AccountCurrentAccountSingleton
                .getAllUserAccounts().getUserAccount(accountId)
                .getNumberOfAllTransactions();
        AccountTransaction lasttransaction = AccountCurrentAccountSingleton
                .getAllUserAccounts().getUserAccount(accountId).getTransaction(
                    numberOfTransactions - 1);

        // Construct the calendar instance with the date of the last transaction
        Calendar lastTransactionCalendar = Calendar.getInstance();
        try {

            lastTransactionCalendar.setLenient(false);
            lastTransactionCalendar.set(lasttransaction.getYear(),
                lasttransaction.getMonth() - 1, lasttransaction.getDay(), 0, 0);

            // this will cause an exception if the date is wrong
            lastTransactionCalendar.get(java.util.Calendar.DAY_OF_MONTH);

        }
        catch(Exception e) {
            showErrorMessage("Unexpected Error. The last transaction Date is not valid. Replace it first.");
            return false;
        }

        // See if the new transaction is before the last transaction date.
        if (inputCalendar.before(lastTransactionCalendar)) {
            showErrorMessage("The Date should be equal or later than the last transaction.");
            return false;
        }

        // Check valuedate
        String valueAuxiliar = valueField.getText();
        if (valueAuxiliar == null)
            valueAuxiliar = new String();
        if (valueAuxiliar.equals("")) {
            showErrorMessage("The Value field is empty");
            return false;
        }

        String valueContent = valueField.getText().trim();

        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(2);

        Number valueTest = null;
        try {
            valueTest = format.parse(valueContent);
        }
        catch(ParseException e1) {
            showErrorMessage("The value field not well formatted.");
            return false;
        }

        // Check Description
        String description = descriptionField.getText();
        if (description == null)
            description = new String();
        if (valueAuxiliar.equals("")) {
            showErrorMessage("The Description field is empty");
            return false;
        }

        // Value in description for deposits
        if (depositButton.isSelected() || savingsButton.isSelected()) {

            Number descriptionValue = null;
            try {
                descriptionValue = AccountValues.getNumberFromText(description);
            }
            catch(NumberNotFoundException e1) {
                showErrorMessage("For this type the description has to contain the value.");
                return false;
            }

            if (descriptionValue.floatValue() != valueTest.floatValue()) {
                showErrorMessage("The value in description must be the same of the transaction value");
                return false;
            }
        }

        return true;
    }

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.control.AccountJInternalFrame#getAccountInternalFrameIdentification()
     */
    public String getAccountInternalFrameIdentification() {

        return accountId + ID_INSERT_APPEND_IDENTIFICATION;
    }

}
