/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import com.carlosbcruz.account.util.AccountMessages;
import com.carlosbcruz.account.util.AccountObserver;
import com.carlosbcruz.account.util.AccountProperties;
import com.carlosbcruz.account.util.AccountSubject;

/**
 * Main class that links all the other classes to form the Account Application
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountMainForm extends JFrame {

    // Percentage of the screen that the com.carlosbcruz.account.control
    // application will use when started;
    private final int windowSizeScreenPercentage = 80; //80%

    // Position in percentage from the com.carlosbcruz.account.control application main window where the
    // divider location will be;
    private final int dividerPosition = 25; //25%

    // Graphical Objects Reference;
    private DefaultTreeModel accountsTreeModel;
    private DefaultMutableTreeNode accauntsRootNode;

    private AccountJDesktopPane desktopPane;
    private JTree tree;

    // Links References
    AccountEventControlImportAction accountEventControlImportAction;
    AccountEventControlLeafTreeSelection accountEventControlLeafTreeSelection;
    AccountEventControlExportAction accountEventControlExportAction;

    /**
     *  Constructor of AccountMainForm. Construct the GUI.
     */
    public AccountMainForm() {

        constructGui();

    }

    /**
     * Reponsible for the basic user interface construction
     */
    private void constructGui() {

        setTitle(AccountMessages.getString("title"));

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Get Icon for the Main Window
        URL imageURL = AccountMainForm.class.getResource(AccountMessages
                .getString("accountImage")); //$NON-NLS-1$
        setIconImage(Toolkit.getDefaultToolkit().getImage(imageURL));

        setJMenuBar(new AccountMainMenu());

        //
        // Calculate the size and position of the main appliction window
        //
        Dimension screenDimension = Toolkit.getDefaultToolkit().getScreenSize();
        // Account main Window width and height when the application is started.
        int windowWidthWhenStarted = screenDimension.width
            * windowSizeScreenPercentage / 100;
        int windowHeightWhenStarted = screenDimension.height
            * windowSizeScreenPercentage / 100;
        // Account main Window position when the application is started.
        int windowXInitialPosition = (screenDimension.width - windowWidthWhenStarted) / 2;
        int windowYInitialPosition = (screenDimension.height - windowHeightWhenStarted) / 2;
        // Set window size and position
        setBounds(windowXInitialPosition, windowYInitialPosition,
            windowWidthWhenStarted, windowHeightWhenStarted);

        //
        // Construct the main objects hierarchy
        //
        accauntsRootNode = new DefaultMutableTreeNode("No Account Selected");

        accountsTreeModel = new DefaultTreeModel(accauntsRootNode);

        tree = new JTree(accountsTreeModel);
        tree.getSelectionModel().setSelectionMode(
            TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);

        JScrollPane listScroolPane = new JScrollPane(tree);

        // Create the desktop to receive internal windows
        desktopPane = new AccountJDesktopPane();

        desktopPane.setBackground(new Color(80, 140, 80));

        tree.addMouseListener(new AccountEventControlTreeDesktopPopupListener(
            tree, desktopPane));

        // Make the split window and add to a Panel
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
            listScroolPane, desktopPane);
        splitPane.setDividerLocation(windowWidthWhenStarted * dividerPosition
            / 100);

        // Glue the SplitPane with the JPanel.		
        JPanel globalPanel = new JPanel();
        globalPanel.setLayout(new BorderLayout());
        globalPanel.add(splitPane, BorderLayout.CENTER);

        AccountActions.getExitAction().addObserver(new AccountObserver() {

            public void update(AccountSubject accountSubject) {

                AccountProperties.saveProperties();
                System.exit(0);
            }
        }

        );
        addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent arg0) {

                AccountProperties.saveProperties();
            }
        });

        globalPanel.add(new AccountToolBar(), BorderLayout.NORTH);

        setContentPane(globalPanel);

        // Application may be displayed now
        setVisible(true);

        // Link the events

        // Instantiate the class that will work 
        // with the Import Action event
        accountEventControlImportAction = new AccountEventControlImportAction(
            this, accauntsRootNode, accountsTreeModel, AccountActions
                    .getImportAction());

        // Instantiate the class that will work 
        // with the Export Action event
        accountEventControlExportAction = new AccountEventControlExportAction(
            this, desktopPane, AccountActions.getExportAction());

        // Instantiate the class that will work 
        // with the Tree Selection event
        accountEventControlLeafTreeSelection = new AccountEventControlLeafTreeSelection(
            tree, desktopPane);

        tree.addTreeSelectionListener(accountEventControlLeafTreeSelection);

    }

    /**
     * Main class. Only instantiate the Account and set a few proprierties.
     */
    public static void main(String[] args) {

        // Create an instance of the main 
        // for the Account Application
        AccountMainForm account = new AccountMainForm();

    }

}
