/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import com.carlosbcruz.account.dynamicmodel.AccountCurrentAccountSingleton;
import com.carlosbcruz.account.dynamicmodel.AccountEvent;
import com.carlosbcruz.account.dynamicmodel.AccountModelListener;
import com.carlosbcruz.account.dynamicmodel.AccountTransaction;
import com.carlosbcruz.account.modeltotals.AccountCalculations;
import com.carlosbcruz.account.modeltotals.AccountTransactionTotal;

/**
 * Store the content of a table
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountTableModel implements TableModel, AccountModelListener {

    private String accountId;
    private int year;

    private AccountTransactionTotal totals[];

    private Vector tableModelListener = new Vector();

    AccountTableModel(String accountId, int year) {

        this.accountId = accountId;
        this.year = year;

        totals = AccountCalculations.getAccountTotals(accountId, year);
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount() {

        return AccountCurrentAccountSingleton.getAllUserAccounts()
                .getUserAccount(accountId).getNumberOfTransactions(year);
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount() {

        return 5;
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#getColumnName(int)
     */
    public String getColumnName(int arg0) {

        switch (arg0) {
            case (0): {
                return "Date";
            }
            case (1): {
                return "Withdraw";
            }
            case (2): {
                return "Account";
            }
            case (3): {
                return "Savings";
            }
            case (4): {
                return "Description";
            }
        }
        return null;
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#getColumnClass(int)
     */
    public Class getColumnClass(int arg0) {

        return String.class;
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#isCellEditable(int, int)
     */
    public boolean isCellEditable(int arg0, int arg1) {

        return false;
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt(int arg1, int arg0) {

        AccountTransaction transaction = AccountCurrentAccountSingleton
                .getAllUserAccounts().getUserAccount(accountId).getTransaction(
                    year, arg1);

        switch (arg0) {
            case (0): {
                return transaction.getDay() + "/" + transaction.getMonth();
            }
            case (1): {
                return String.valueOf(transaction.getValue());
            }
            case (2): {
                return String.valueOf(totals[arg1].getAccountTotal());
            }
            case (3): {
                return String.valueOf(totals[arg1].getSavingsTotal());
            }
            case (4): {
                return transaction.getDescription();
            }
        }

        return null;
    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
     */
    public void setValueAt(Object arg0, int arg1, int arg2) {

    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#addTableModelListener(javax.swing.event.TableModelListener)
     */
    public void addTableModelListener(TableModelListener arg0) {

        tableModelListener.add(arg0);

    }

    /* (no-Javadoc)
     * @see javax.swing.table.TableModel#removeTableModelListener(javax.swing.event.TableModelListener)
     */
    public void removeTableModelListener(TableModelListener arg0) {

        tableModelListener.remove(arg0);
    }

    /**
     * Notify the Tree that the model has changed
     */
    private void notifyModelChanged() {

        Enumeration e = tableModelListener.elements();
        while (e.hasMoreElements()) {

            TableModelListener listener = (TableModelListener)e.nextElement();
            TableModelEvent event = new TableModelEvent(this);

            listener.tableChanged(event);
        }

    }

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.dynamicmodel.AccountModelListener#accountModelChanged(com.carlosbcruz.account.dynamicmodel.AccountEvent)
     */
    public void accountModelChanged(AccountEvent e) {

        totals = AccountCalculations.getAccountTotals(accountId, year);

        if (e.getAccountId().equals(accountId) && e.getYear() >= year)
            notifyModelChanged();

    }

}
