/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

/**
 * Parsers the specific Citibank bank report
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Citibank implements AccountParser {

    public class CitibankBean implements AccountTransactionBean {

        private String day;
        private String month;
        private String year;
        private String description;
        private String value;
        private boolean withdraw;

        public CitibankBean() {

            this.day = "00";
            this.month = "00";
            this.year = "0000";
            this.description = "0";
            this.value = "0";
            this.withdraw = true;

        }

        public CitibankBean(String day, String month, String year,
            String description, String value, boolean withdraw) {

            this.day = day;
            this.month = month;
            this.year = year;
            this.description = description;
            this.value = value;
            this.withdraw = withdraw;

        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getDay()
         */
        public String getDay() {

            return day;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getMonth()
         */
        public String getMonth() {

            return month;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getYear()
         */
        public String getYear() {

            return year;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getValue()
         */
        public String getValue() {

            return value;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getDescription()
         */
        public String getDescription() {

            return description;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#isWidthdraw()
         */
        public boolean isWithdraw() {

            return withdraw;
        }
    }

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.control.AccountParser#parseFirstTransactionLine(java.lang.String)
     */
    public AccountTransactionBean parseFirstTransactionLine(String report) {

        String day = report.substring(0, 2);

        String month = report.substring(3, 5);

        String year = report.substring(6, 10);

        int lineEndPosition = report.indexOf("\n");

        if (lineEndPosition == -1)
            return new CitibankBean();

        // Find Last number position
        int lastNumber = 0;
        for (int i = lineEndPosition; i > 0; i--) {
            if (Character.isDigit(report.charAt(i))) {
                lastNumber = i;
                break;
            }
        }

        int spacePosition = 0;
        for (int i = lastNumber; i > 0; i--) {
            if (!Character.isDigit(report.charAt(i)) && report.charAt(i) != ','
                && report.charAt(i) != '.') {
                spacePosition = i;
                break;
            }
        }

        String value = report.substring(spacePosition, lastNumber).trim();

        String description = report.substring(11, spacePosition).trim();

        int minusPosition = value.indexOf("-");
        boolean withdraw = minusPosition != -1;

        if (withdraw)
            value = value.substring(minusPosition + 1, value.length());

        return new CitibankBean(day, month, String.valueOf(year), description,
            value, withdraw);
    }

}
