/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.util.Calendar;

/**
 * Parsers the specific Itau bank report
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Itau2 implements AccountParser {

    /**
     * Store a transaction
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    public class ItauBean implements AccountTransactionBean {

        private String day;
        private String month;
        private String year;
        private String description;
        private String value;
        private boolean withdraw;

        public ItauBean() {

            this.day = "00";
            this.month = "00";
            this.year = "0000";
            this.description = "0";
            this.value = "0";
            this.withdraw = true;

        }

        public ItauBean(String day, String month, String year,
            String description, String value, boolean withdraw) {

            this.day = day;
            this.month = month;
            this.year = year;
            this.description = description;
            this.value = value;
            this.withdraw = withdraw;

        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getDay()
         */
        public String getDay() {

            return day;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getMonth()
         */
        public String getMonth() {

            return month;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getYear()
         */
        public String getYear() {

            return year;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getValue()
         */
        public String getValue() {

            return value;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getDescription()
         */
        public String getDescription() {

            return description;
        }

        /* (no-Javadoc)
         * @see com.carlosbcruz.account.control.AccountTransactionBean#isWidthdraw()
         */
        public boolean isWithdraw() {

            return withdraw;
        }
    }

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.control.AccountParser#parseFirstTransactionLine(java.lang.String)
     */
    public AccountTransactionBean parseFirstTransactionLine(String report) {

        int lineEndPosition = report.indexOf("\n");

        if (lineEndPosition == -1)
            return new ItauBean();

        String day = report.substring(0, 2);

        String month = report.substring(3, 5);

        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);

        String description = report.substring(6, lineEndPosition);

        String sign = description.substring(description.length() - 2,
            description.length() - 1);
        boolean withdraw = sign.equals("-");
        description = description.substring(0, description.length() - 3).trim();

        String value = "0";
        for (int i = description.length() - 1; i > 0; i--)
            if (description.charAt(i) == ' ') {
                value = description.substring(i + 1, description.length());
                description = description.substring(0, i + 1).trim();
                break;
            }

        return new ItauBean(day, month, String.valueOf(year), description,
            value, withdraw);
    }

}
