/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.control;

import java.util.Calendar;

/**
 * Parsers the specific Itau bank report
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Itau3 implements AccountParser {

    /**
     * Store a transaction
     * 
     * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
     */
    public class ItauBean implements AccountTransactionBean {

        private String day;

        private String month;

        private String year;

        private String description;

        private String value;

        private boolean withdraw;

        public ItauBean() {

            this.day = "00";
            this.month = "00";
            this.year = "0000";
            this.description = "0";
            this.value = "0";
            this.withdraw = true;

        }

        public ItauBean(String day, String month, String year,
            String description, String value, boolean withdraw) {

            this.day = day;
            this.month = month;
            this.year = year;
            this.description = description;
            this.value = value;
            this.withdraw = withdraw;

        }

        /*
         * (no-Javadoc)
         * 
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getDay()
         */
        public String getDay() {

            return day;
        }

        /*
         * (no-Javadoc)
         * 
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getMonth()
         */
        public String getMonth() {

            return month;
        }

        /*
         * (no-Javadoc)
         * 
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getYear()
         */
        public String getYear() {

            return year;
        }

        /*
         * (no-Javadoc)
         * 
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getValue()
         */
        public String getValue() {

            return value;
        }

        /*
         * (no-Javadoc)
         * 
         * @see com.carlosbcruz.account.control.AccountTransactionBean#getDescription()
         */
        public String getDescription() {

            return description;
        }

        /*
         * (no-Javadoc)
         * 
         * @see com.carlosbcruz.account.control.AccountTransactionBean#isWidthdraw()
         */
        public boolean isWithdraw() {

            return withdraw;
        }

        /*
         * (no-Javadoc)
         * 
         * @see java.lang.Object#toString()
         */
        public String toString() {

            String out = "ItauBean(" + day + "/" + month + "/" + year + ","
                + description + ",#" + value + "#,"
                + (withdraw ? "withdraw" : "deposit") + ")";
            return out;
        }
    }

    /*
     * (no-Javadoc)
     * 
     * @see com.carlosbcruz.account.control.AccountParser#parseFirstTransactionLine(java.lang.String)
     */
    public AccountTransactionBean parseFirstTransactionLine(String report) {

        int lineEndPosition = report.indexOf("\n");

        if (lineEndPosition == -1)
            return new ItauBean();

        report = report.substring(0, lineEndPosition).trim();

        String day = report.substring(0, 2);

        String month = report.substring(3, 5);

        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);

        String description = report.substring(6, report.length());

        // See if it is a deposit or widthdraw
        boolean withdraw = false;
        for (int i = description.length() - 1; i > 0; i--)
            if (description.charAt(i) != ' ') {
                if (description.charAt(i) == '-')
                    withdraw = true;
                break;
            }

        // Get value		
        String value = "0";
        int end = -1;
        int start = -1;
        boolean hasValueStarted = false;
        for (int i = description.length() - 1; i > 0; i--)
            if (Character.isDigit(description.charAt(i))
                || description.charAt(i) == ',' || description.charAt(i) == '.') {
                if (!hasValueStarted) {
                    hasValueStarted = true;
                    end = i;
                }
            } else
                if (hasValueStarted) {
                    start = i;
                    break;
                }

        value = description.substring(start + 1, end + 1);
        description = description.substring(3, start - 1).trim();

        return new ItauBean(day, month, String.valueOf(year), description,
            value, withdraw);
    }

}
