/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.dynamicmodel;

import java.util.*;

/** 
 * Main model class for Account application. It has all the Users Account and Checks Groups.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountAllUserAccounts {

    private String owner;

    private List userAccount = new LinkedList();

    private List checkGroups = new LinkedList();

    private List accountComment = new LinkedList();

    /**
     * Returns the owner.
     * @return The Owner
     */
    public String getOwner() {

        return owner;
    }

    /**
     * Sets the owner.
     * @param owner The owner to set
     */
    public void setOwner(String owner) {

        this.owner = owner;
    }

    /**
     * Inform how manuy accounts there is
     * @return The number of accounts
     */
    public int getNumberOfUserAccounts() {

        return userAccount.size();
    }

    /**
     * Provide the user com.carlosbcruz.account.control based on its position
     * @param position Account position
     * @return The com.carlosbcruz.account.control requested
     */
    public AccountUserAccount getUserAccount(int position) {

        return (AccountUserAccount)userAccount.get(position);
    }

    /**
     * Returns a specific userAccount.
     * @return The Specific User Account
     */
    public AccountUserAccount getUserAccount(String accountId) {

        Iterator i = userAccount.iterator();
        while (i.hasNext()) {
            AccountUserAccount userAccount = (AccountUserAccount)i.next();

            if (userAccount.getAccountId().equals(accountId))
                return userAccount;
        }

        return null;
    }

    /**
     * Add an user com.carlosbcruz.account.control
     * @param accountUserAccount The user com.carlosbcruz.account.control to be inserted
     */
    public void addUserAccount(AccountUserAccount accountUserAccount) {

        userAccount.add(accountUserAccount);
    }

    /**
     * Inform the Number of Check Groups
     * @return the number of check groups
     */
    public int getNumberOfCheckGroups() {

        return checkGroups.size();
    }

    /**
     * Inform the Check Group by position. The first position is zero.
     * @param position the specified position
     * @return The Check Groups required.
     */
    public AccountCheckGroup getCheckGroup(int position) {

        return (AccountCheckGroup)checkGroups.get(position);
    }

    /**
     * Returns a specific checkGroups.
     * @return The specific Check Groups
     */
    public AccountCheckGroup getCheckGroup(String checkGroupIdentification) {

        Iterator i = checkGroups.iterator();
        while (i.hasNext()) {
            AccountCheckGroup result = (AccountCheckGroup)i.next();
            if (result.getCheckGroupIdentification().equals(
                checkGroupIdentification))
                return result;
        }

        return null;
    }

    /**
     * Insert a check group to the accounts
     * @param checkGroup The check group to be inserted
     */
    public void addCheckGroup(AccountCheckGroup checkGroup) {

        checkGroups.add(checkGroup);
    }

    /**
     * Inform how many comments there are
     * @return The number of comments
     */
    public int getNumberOfComments() {

        return accountComment.size();
    }

    /**
     * Inform the comment to a specified position. The first comment is in
     * position zero
     * @param position The comment position
     * @return The comment required
     */
    public AccountComment getAccountComment(int position) {

        return (AccountComment)accountComment.get(position);
    }

    /**
     * Returns a specific comment.
     * @return The specific comment
     */
    public AccountComment getAccountComments(String commentIdentification) {

        Iterator i = accountComment.iterator();
        while (i.hasNext()) {
            AccountComment result = (AccountComment)i.next();
            if (result.getCommentId().equals(commentIdentification))
                return result;
        }

        return null;
    }

    /**
     * Add a Comment to the Accounts
     * @param comment The comment to be inserted
     */
    public void addComment(AccountComment comment) {

        accountComment.add(comment);
    }

}
