/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.dynamicmodel;

/**
 * Represents a transaction in an com.carlosbcruz.account.control. It can be a withdraw, 
 * deposit to a current com.carlosbcruz.account.control, deposit to a savings com.carlosbcruz.account.control or
 * a move between current com.carlosbcruz.account.control and savings. A transaction also
 * has a date, a value and a description.
 *
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountTransaction {

    // Possible types of transactions
    public final static int WITHDRAW = 0;
    public final static int DEPOSIT_CURRENT_ACCOUNT = 1;
    public final static int DEPOSIT_SAVINGS_ACCOUNT = 2;
    public final static int MOVE_FROM_CURRENT_ACCOUNT_TO_SAVINGS = 3;
    public final static int MOVE_FROM_SAVINGS_TO_CURRENT_ACCOUNT = 4;
    // The type of the transaction. It can be one of the above types.
    private int type;

    // Date that the transaction occured
    private int day;
    private int month;
    private int year;

    // Value of the transaction
    private float value;

    // Transaction description
    private String description;

    /**
     * Returns the type.
     * @return The transaction type
     */
    public int getType() {

        return type;
    }

    /**
     * Returns the value.
     * @return The transaction value
     */
    public float getValue() {

        return value;
    }

    /**
     * Sets the type.
     * @param type The type to set
     */
    public void setType(int type) {

        this.type = type;
    }

    /**
     * Returns the day.
     * @return The transaction date
     */
    public int getDay() {

        return day;
    }

    /**
     * Sets the day.
     * @param day The day to set
     */
    public void setDay(int day) {

        this.day = day;
    }

    /**
     * Returns the month.
     * @return The transction month
     */
    public int getMonth() {

        return month;
    }

    /**
     * Sets the month.
     * @param month The month to set
     */
    public void setMonth(int month) {

        this.month = month;
    }

    /**
     * Returns the year.
     * @return The transaction year
     */
    public int getYear() {

        return year;
    }

    /**
     * Sets the value.
     * @param value The value to set
     */
    public void setValue(float value) {

        this.value = value;
    }

    /**
     * Sets the year.
     * @param year The year to set
     */
    public void setYear(int year) {

        this.year = year;
    }

    /**
     * Returns the description.
     * @return The transaction description
     */
    public String getDescription() {

        return description;
    }

    /**
     * Sets the description.
     * @param description The description to set
     */
    public void setDescription(String description) {

        this.description = description;
    }
}
