/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.modelpersistence;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.NumberFormat;

import com.carlosbcruz.account.dynamicmodel.AccountCheck;
import com.carlosbcruz.account.dynamicmodel.AccountCheckGroup;
import com.carlosbcruz.account.dynamicmodel.AccountComment;
import com.carlosbcruz.account.dynamicmodel.AccountCurrentAccountSingleton;
import com.carlosbcruz.account.dynamicmodel.AccountModelPersistence;
import com.carlosbcruz.account.dynamicmodel.AccountTransaction;
import com.carlosbcruz.account.dynamicmodel.AccountUserAccount;
import com.carlosbcruz.account.modeltotals.AccountCalculations;
import com.carlosbcruz.account.modeltotals.AccountTransactionTotal;

/**
 * Controls the output of information from a file
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountWriter implements AccountModelPersistence {

    private FileWriter fileWriter;
    private BufferedWriter bufferedWriter;

    private final String NEW_LINE = System.getProperty("line.separator");

    /* (no-Javadoc)
     * @see com.carlosbcruz.account.dynamicmodel.AccountModelPersistence#WriteAllUserAccounts(java.lang.String)
     */
    public void WriteAllUserAccounts(String fileName)
        throws AccountFileException {

        try {
            // Open the output file
            fileWriter = new FileWriter(fileName);
            bufferedWriter = new BufferedWriter(fileWriter);

            // Write the version
            bufferedWriter.write(AccountReader.FILE_TYPE_VERSION + NEW_LINE);

            // Write the owner
            bufferedWriter.write(AccountCurrentAccountSingleton
                    .getAllUserAccounts().getOwner()
                + NEW_LINE);

            // Write how many accounts there are
            int numberOfAccounts = AccountCurrentAccountSingleton
                    .getAllUserAccounts().getNumberOfUserAccounts();

            bufferedWriter.write(String.valueOf(numberOfAccounts) + NEW_LINE);

            // Write all the com.carlosbcruz.account.control titles
            for (int i = 0; i < numberOfAccounts; i++) {

                AccountUserAccount account = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getUserAccount(i);

                bufferedWriter.write(String.valueOf(String.valueOf(i + 1))
                    + "-" + account.getAccountId() + NEW_LINE);
            }

            // Write the total of check groups
            int numberOfCheckGroups = AccountCurrentAccountSingleton
                    .getAllUserAccounts().getNumberOfCheckGroups();
            bufferedWriter
                    .write(String.valueOf(numberOfCheckGroups) + NEW_LINE);

            // Write all the check groups titles
            for (int i = 0; i < numberOfCheckGroups; i++) {
                AccountCheckGroup checkGroup = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getCheckGroup(i);
                bufferedWriter
                        .write(String.valueOf(String.valueOf(i + 1)) + "-"
                            + checkGroup.getCheckGroupIdentification()
                            + NEW_LINE);
            }

            // Write the total of comments
            int numberOfComments = AccountCurrentAccountSingleton
                    .getAllUserAccounts().getNumberOfComments();

            bufferedWriter.write(String.valueOf(numberOfComments) + NEW_LINE);

            // Write all the comments titles
            for (int i = 0; i < numberOfComments; i++) {

                AccountComment comment = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getAccountComment(i);

                bufferedWriter.write(String.valueOf(String.valueOf(i + 1))
                    + "-" + comment.getCommentId() + NEW_LINE);
            }

            // Write all the com.carlosbcruz.account.control transactions
            for (int i = 0; i < numberOfAccounts; i++) {

                AccountUserAccount account = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getUserAccount(i);

                // Write the beginning of com.carlosbcruz.account.control transactions identification
                bufferedWriter.write(account.getAccountId() + NEW_LINE);

                AccountUserAccount userAccount = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getUserAccount(
                            account.getAccountId());

                AccountTransactionTotal transactionsTotal[] = AccountCalculations
                        .getAccountTotals(account.getAccountId());

                int lastYear = -1;
                float lastSavings = 0;
                for (int j = 0; j < userAccount.getNumberOfAllTransactions(); j++) {

                    AccountTransaction transaction = userAccount
                            .getTransaction(j);

                    // if it is a new year than write the year
                    if (transaction.getYear() != lastYear) {
                        bufferedWriter.write(String.valueOf(transaction
                                .getYear())
                            + NEW_LINE);
                        lastYear = transaction.getYear();
                    }
                    // Write the date

                    // It is necessary two digit day
                    String day = transaction.getDay() < 10 ? "0"
                        + transaction.getDay() : String.valueOf(transaction
                            .getDay());
                    // it is necessary two digit month
                    String month = transaction.getMonth() < 10 ? "0"
                        + transaction.getMonth() : String.valueOf(transaction
                            .getMonth());
                    bufferedWriter.write(day + "/" + month);

                    NumberFormat numberFormat = NumberFormat
                            .getNumberInstance();
                    numberFormat.setMinimumFractionDigits(2);
                    numberFormat.setMaximumFractionDigits(2);

                    // Write the transaction value in 11 available spaces
                    if (transaction.getType() == AccountTransaction.WITHDRAW) {
                        String value = numberFormat.format(transaction
                                .getValue());
                        int missingSpaces = 11 - value.length();
                        for (int k = 0; k < missingSpaces; k++)
                            bufferedWriter.write(" ");
                        bufferedWriter.write(value);
                    } else
                        bufferedWriter.write("           ");

                    // Write the current com.carlosbcruz.account.control value in 11 available spaces

                    if (transaction.getType() != AccountTransaction.DEPOSIT_SAVINGS_ACCOUNT) {
                        String accountValue = numberFormat
                                .format(transactionsTotal[j].getAccountTotal());
                        int missingSpaces = 11 - accountValue.length();
                        for (int k = 0; k < missingSpaces; k++)
                            bufferedWriter.write(" ");
                        bufferedWriter.write(accountValue);
                    } else
                        bufferedWriter.write("           ");

                    // Write the savings value in 11 available spaces
                    if (transactionsTotal[j].getSavingsTotal() != lastSavings) {
                        lastSavings = transactionsTotal[j].getSavingsTotal();

                        String savingsValue = numberFormat
                                .format(transactionsTotal[j].getSavingsTotal());
                        int missingSpaces = 11 - savingsValue.length();
                        for (int k = 0; k < missingSpaces; k++)
                            bufferedWriter.write(" ");
                        bufferedWriter.write(savingsValue);
                    } else
                        bufferedWriter.write("           ");

                    // Write Description
                    if (transaction.getType() != AccountTransaction.MOVE_FROM_CURRENT_ACCOUNT_TO_SAVINGS
                        && transaction.getType() != AccountTransaction.MOVE_FROM_SAVINGS_TO_CURRENT_ACCOUNT)
                        bufferedWriter.write(" " + transaction.getDescription()
                            + NEW_LINE);
                    else
                        bufferedWriter.write(" " + NEW_LINE);
                }

                // Write the end of the com.carlosbcruz.account.control transactions identification
                bufferedWriter
                        .write("FIM " + account.getAccountId() + NEW_LINE);

            }

            // Write all the com.carlosbcruz.account.control transactions
            for (int i = 0; i < numberOfCheckGroups; i++) {

                AccountCheckGroup checkGroup = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getCheckGroup(i);

                // Write the beginning of com.carlosbcruz.account.control transactions identification
                bufferedWriter.write(checkGroup.getCheckGroupIdentification()
                    + NEW_LINE);

                // Write all the checks
                int numberOfChecks = checkGroup.getNumberOfChecks();

                for (int k = 0; k < numberOfChecks; k++) {

                    AccountCheck check = checkGroup.getCheck(k);

                    // Write a counter
                    String counter = String.valueOf(k + 1);
                    int missingSpaces = 4 - counter.length();
                    for (int l = 0; l < missingSpaces; l++)
                        bufferedWriter.write(" ");
                    bufferedWriter.write(counter);

                    //	Write a check identification
                    String checkIdentification = check.getCheckIdentification()
                            .trim();
                    missingSpaces = 8 - checkIdentification.length();
                    for (int l = 0; l < missingSpaces; l++)
                        bufferedWriter.write(" ");
                    bufferedWriter.write(checkIdentification);

                    NumberFormat numberFormat = NumberFormat
                            .getNumberInstance();
                    numberFormat.setMinimumFractionDigits(2);
                    numberFormat.setMaximumFractionDigits(2);

                    // Write value
                    String value = numberFormat.format(check.getValue());
                    missingSpaces = 10 - value.length();
                    for (int l = 0; l < missingSpaces; l++)
                        bufferedWriter.write(" ");
                    bufferedWriter.write(value);

                    // Write description
                    String description = check.getDescription().trim();
                    missingSpaces = 40 - description.length();
                    bufferedWriter.write(" " + description);
                    for (int l = 0; l < missingSpaces; l++)
                        bufferedWriter.write(" ");

                    // Write Status
                    String status = check.getStatus();

                    if (status != null) {
                        status = status.trim();
                        missingSpaces = 10 - status.length();
                        for (int l = 0; l < missingSpaces; l++)
                            bufferedWriter.write(" ");
                        bufferedWriter.write(" " + status + NEW_LINE);
                    } else
                        bufferedWriter.write(NEW_LINE);
                }

                // Write the end of the com.carlosbcruz.account.control transactions identification
                bufferedWriter.write("FIM "
                    + checkGroup.getCheckGroupIdentification() + NEW_LINE);

            }

            // Write all the comments
            for (int i = 0; i < numberOfComments; i++) {

                AccountComment comment = AccountCurrentAccountSingleton
                        .getAllUserAccounts().getAccountComment(i);

                // Write the beginning of a comment identification
                bufferedWriter.write(comment.getCommentId() + NEW_LINE);

                // Write the comment
                bufferedWriter.write(comment.getComments().toString());

                // Write the end of the comment identification
                bufferedWriter
                        .write("FIM " + comment.getCommentId() + NEW_LINE);
            }

            // Close the file
            bufferedWriter.close();
            fileWriter.close();

        }
        catch(IOException e) {
            throw new AccountFileException("IO Error while creating file "
                + fileName);
        }

    }
}
