/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.modeltotals;

import com.carlosbcruz.account.dynamicmodel.*;
import com.carlosbcruz.account.util.AccountValues;

/**
 * Helps in calculate the totals of both current com.carlosbcruz.account.control and savings com.carlosbcruz.account.control
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountCalculations {

    /**
     * Calculate all the totals corresponding to each transactions of an com.carlosbcruz.account.control
     * @param accountId Account Identification
     * @return AccountTransactionTotal[] The total corresponding to each transaction
     */
    public static AccountTransactionTotal[] getAccountTotals(String accountId) {

        // Get user Account
        AccountUserAccount userAccount = AccountCurrentAccountSingleton
                .getAllUserAccounts().getUserAccount(accountId);

        // Create the return array of all the totals
        AccountTransactionTotal transactionsTotal[] = new AccountTransactionTotal[userAccount
                .getNumberOfAllTransactions()];

        // Follow all the transactions and make the corresponding
        // calculation based on the transaction type.
        float accountTotal = 0, savingsTotal = 0;
        for (int i = 0; i < transactionsTotal.length; i++) {

            AccountTransaction transaction = userAccount.getTransaction(i);

            // Make the calculations of the totals
            switch (transaction.getType()) {

                case (AccountTransaction.DEPOSIT_CURRENT_ACCOUNT): {
                    accountTotal += transaction.getValue();
                    // Garantee two digit decimal on float
                    accountTotal = AccountValues
                            .getTwoDecimalFloat(accountTotal);

                    break;
                }

                case (AccountTransaction.DEPOSIT_SAVINGS_ACCOUNT): {
                    savingsTotal += transaction.getValue();
                    // Garantee two digit decimal on float
                    savingsTotal = AccountValues
                            .getTwoDecimalFloat(savingsTotal);

                    break;
                }
                case (AccountTransaction.MOVE_FROM_CURRENT_ACCOUNT_TO_SAVINGS): {
                    accountTotal -= transaction.getValue();
                    // Garantee two digit decimal on float
                    accountTotal = AccountValues
                            .getTwoDecimalFloat(accountTotal);

                    savingsTotal += transaction.getValue();
                    // Garantee two digit decimal on float
                    savingsTotal = AccountValues
                            .getTwoDecimalFloat(savingsTotal);

                    break;
                }
                case (AccountTransaction.MOVE_FROM_SAVINGS_TO_CURRENT_ACCOUNT): {
                    accountTotal += transaction.getValue();
                    // Garantee two digit decimal on float
                    accountTotal = AccountValues
                            .getTwoDecimalFloat(accountTotal);

                    savingsTotal -= transaction.getValue();
                    // Garantee two digit decimal on float
                    savingsTotal = AccountValues
                            .getTwoDecimalFloat(savingsTotal);

                    break;
                }
                case (AccountTransaction.WITHDRAW): {
                    accountTotal -= transaction.getValue();
                    // Garantee two digit decimal on float
                    accountTotal = AccountValues
                            .getTwoDecimalFloat(accountTotal);

                    break;
                }
                default: {
                    // Not possible
                }
            }

            transactionsTotal[i] = new AccountTransactionTotal();

            transactionsTotal[i].setAccountTotal(accountTotal);
            transactionsTotal[i].setSavingsTotal(savingsTotal);

        }

        return transactionsTotal;
    }

    /**
     * Calculate all the totals corresponding to each transactions of an com.carlosbcruz.account.control
     * to a specif year
     * @param accountId Account Identification
     * @param year Specifc Year
     * @return AccountTransactionTotal[] The total corresponding to each transaction
     */
    public static AccountTransactionTotal[] getAccountTotals(String accountId,
        int year) {

        AccountUserAccount userAccount = AccountCurrentAccountSingleton
                .getAllUserAccounts().getUserAccount(accountId);

        int firstTransaction = userAccount.getPositionOfFirstTransactions(year);
        int countTransaction = userAccount.getNumberOfTransactions(year);

        // Get the total of all the transactions for all the years
        AccountTransactionTotal transactionsTotal[] = getAccountTotals(accountId);

        AccountTransactionTotal transactionsReturn[] = new AccountTransactionTotal[countTransaction];

        // Transfer only the part for the specified year
        for (int i = firstTransaction, j = 0; i < (firstTransaction + countTransaction); i++, j++) {

            transactionsReturn[j] = new AccountTransactionTotal();

            transactionsReturn[j].setAccountTotal(transactionsTotal[i]
                    .getAccountTotal());
            transactionsReturn[j].setSavingsTotal(transactionsTotal[i]
                    .getSavingsTotal());
        }
        return transactionsReturn;
    }

}