/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.util;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Icon;

/**
 * Provides the AbstractAction with an enhancement with the use of a Decorator
 * Design Patter. A new Account Abstract Action Decorator allows listeners to
 * be attached to Actions. Whenever an Action is Performed all the listeners
 * will be notified behind the scenes.
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountAbstractActionDecorator extends AbstractAction implements
    AccountSubject {

    // Listeners registered to the Action
    Collection listeners = new Vector();

    /**
     * Replicate the constructor specified at ActionSubject class.
     * @param name The name of the Action
     * @param icon The icon that represents the Action
     */
    public AccountAbstractActionDecorator(String name, Icon icon) {

        super(name, icon);
    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public synchronized void actionPerformed(ActionEvent e) {

        notifyChange();
    }

    /* (non-Javadoc)
     * @see com.carlosbcruz.account.util.AccountSubject#addObserver(com.carlosbcruz.account.util.AccountObserver)
     */
    public synchronized void addObserver(AccountObserver observer) {

        listeners.add(observer);
    }

    /* (non-Javadoc)
     * @see com.carlosbcruz.account.util.AccountSubject#removeObserver(com.carlosbcruz.account.util.AccountObserver)
     */
    public synchronized void removeObserver(AccountObserver observer) {

        listeners.remove(observer);
    }

    /* (non-Javadoc)
     * @see com.carlosbcruz.account.util.AccountSubject#notifyChange()
     */
    public synchronized void notifyChange() {

        Iterator listenersIterator = listeners.iterator();
        // Follow all the listeners and notify of an Action occurence.
        while (listenersIterator.hasNext()) {
            AccountObserver observer = (AccountObserver)listenersIterator
                    .next();
            observer.update(this);
        }

    }

}