/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.account.util;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

/**
 * Helps in mantain the values under defined limits
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class AccountValues {

    /**
     * Eliminate the rounding problem and sick to two decimals
     * This is necessary because of rouding problems of floats
     * @param value The value. 
     * @return  The value with at maximum of two decimals. 
     */
    public static float getTwoDecimalFloat(float value) {

        NumberFormat format = NumberFormat.getInstance(Locale.ENGLISH);
        format.setMaximumFractionDigits(2);

        String valueFormated = null;
        Number resultValue = null;
        try {
            valueFormated = format.format(value);
        }
        catch(RuntimeException e) {

            e.printStackTrace();
        }
        try {
            resultValue = format.parse(valueFormated);
        }
        catch(ParseException e1) {

            e1.printStackTrace();
        }

        return (resultValue.floatValue());
    }

    /**
     * Extract the number that is inserted inside a String
     * @param text The text to be searched
     * @return The value
     * @throws NumberNotFoundException
     */
    public static Number getNumberFromText(String text)
        throws NumberNotFoundException {

        if (text == null)
            throw new NumberNotFoundException("No text");

        //
        // Find number limits in the description string
        //
        int firstNumber = 0;
        for (int i = 0; i < text.length(); i++) {
            if (text.charAt(i) >= '0' && text.charAt(i) <= '9') {
                firstNumber = i;
                break;
            }
        }

        int lastNumber = 0;
        for (int i = text.length() - 1; i >= 0; i--) {
            if (text.charAt(i) >= '0' && text.charAt(i) <= '9') {
                lastNumber = i;
                break;
            }
        }

        //
        // Check if it is a valid limit
        //
        if (firstNumber >= lastNumber) {
            throw new NumberNotFoundException(
                "Could not find value from Text. Text: \"" + text + "\"");
        }

        //
        // Parse the number.
        //	
        String parsedValue = text.substring(firstNumber, lastNumber + 1);
        NumberFormat formatter = NumberFormat.getInstance();

        Number value = null;
        try {
            value = formatter.parse(parsedValue);
        }
        catch(ParseException e) {
            throw new NumberNotFoundException(
                "Could not parse value from description. \"" + text + "\"");
        }

        return value;

    }
}